{
  description = "Zentralwerk network";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs";
    # TODO: nixpkgs.url = "github:NixOS/nixpkgs/nixos-21.05";
    # `nix flake update --override-flake zentralwerk-network-key git+file:///...`
    # to provide the GPG secret key
    zentralwerk-network-key.url = "git+https://gitea.c3d2.de/zentralwerk/network.git?dir=nix/key";
  };

  outputs = inputs@{ self, nixpkgs, zentralwerk-network-key }:
    let
      system = "x86_64-linux";
      systems = [ system ];
      forAllSystems = nixpkgs.lib.genAttrs systems;

      nixosConfig = name:
        self.lib.nixosSystem {
          inherit system;
          modules = [ self.nixosModule ];
          specialArgs.hostName = name;
          specialArgs.lib = self.lib;
          specialArgs.self = self;
          specialArgs.inputs = inputs;
        };
    in {
      # Config, and utilities
      lib = nixpkgs.lib.extend (final: prev:
        import ./nix/lib {
          inherit self;
          inherit (zentralwerk-network-key.lib) gpgKey;
          pkgs = nixpkgs.legacyPackages.x86_64-linux;
        });

      # Everything that can be built locally outside of NixOS
      packages = forAllSystems (system:
        import ./nix/pkgs { inherit self nixpkgs system; }
      );

      # Configuration for nixosConfigurations
      # (see nix/nixos-module/default.nix)
      nixosModule = { ... }: {
        imports = [ ./nix/nixos-module ];
      };

      # NixOS host systems (servers, and containers)
      nixosConfigurations =
        builtins.mapAttrs (hostName: _: nixosConfig hostName) (
          nixpkgs.lib.filterAttrs (_: { role, ... }:
            builtins.elem role [ "server" "container" ]
          ) self.lib.config.site.hosts
        );

      # For `nix flake check`, and Hydra
      checks = self.packages;

      hydraJobs =
        with self.lib;
        let
          exportFileWrapper = pkg:
            hydraJob (
              nixpkgs.legacyPackages.${system}.runCommand "${pkg.name}-exported" {
                src = pkg;
              } ''
                NAME="${pkg.name}"
                mkdir -p $out/nix-support
                ln -s ${pkg} $out/$NAME
                echo file source-dist $out/$NAME >> $out/nix-support/hydra-build-products
              ''
            );
          wrappers = {
            "all-device-scripts" = pkg:
              hydraJob pkg;

            "export-config" = exportFileWrapper;
            ".*-pillar" = exportFileWrapper;
            "switch-.*" = exportFileWrapper;
            "ap.*" = exportFileWrapper;
          };
        in
          builtins.mapAttrs (name: pkg:
            builtins.foldl' (result: wrapperName:
              if builtins.match wrapperName name != null
              then wrappers.${wrapperName} pkg
              else result
            ) pkg (builtins.attrNames wrappers)
          ) self.packages.${system};
    };
}
